<?php
/**
 * Example to use myMVC.
 * This program just demonstrate how to create
 * a login form, authenticate user, check data from
 * a form and read data from database.
 * 
 * Author: Luca Bertoncello <lucabert@lucabert.de>
 */

/**
 * Example for an action that receives data from user
 * and validate them.
 */
class MainAction extends myMVC_BaseAction
{
  public function __construct($request, $session)
  {
    parent::__construct($request, $session);
// Setting validation function
    $this->setCheckFunction('POST', 'validatePost');
  }

  /**
   * Returns the token used to identify the current Action
   * 
   * @return string                 The token
   */
  public static function getActionToken()
  {
    return 'main';
  }

  /**
   * Returns whether this action should appear in the menu (created by controller)
   * 
   * @return boolean                true if the action should appear in menu
   */
  public static function isMenuAction()
  {
// Yes, this action can be called from menu!
    return true;
  }

  /**
   * Returns the string to be displayed in the menu to identify the current action.
   * Only necessary if isMenuAction() returns true
   * 
   * @return string                 The menu item
   */
  public static function getActionMenuName()
  {
// This string will be displayed in menu.
// Please note, that all strings, here, use GetText!
    return _('Main page');
  }

  /**
   * Returns whether the current user is allowed to call the current Action
   * 
   * @return boolean                true if the user may call this Action, false otherwise
   */
  public static function hasUserRightForAction()
  {
// And yes: the user can call this action! :D
    return true;
  }

  /**
   * Validate the data from user, coming as POST (form submit).
   * 
   * @return boolean                true if data are valid, false otherwise
   */
  public function validatePost()
  {
// Definitions of the checks
    $definitions = array(
        'text' => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'minlen' => 3, 'empty' => false),
        'bool' => array('type' => myMVC_BaseAction::$VALIDATE_BOOLEAN, 'empty' => false),
        'number' => array('type' => myMVC_BaseAction::$VALIDATE_NUMBER, 'empty' => false, 'range' => array(1, 2, 3, 5)),
        'email' => array('type' => myMVC_BaseAction::$VALIDATE_EMAIL, 'empty' => true, 'range' => array('lucabert@lucabert.de', 'lucabert@lucabert.com')),
        'web' => array('type' => myMVC_BaseAction::$VALIDATE_WEB, 'empty' => false),
        'domain' => array('type' => myMVC_BaseAction::$VALIDATE_WEB, 'empty' => true, 'justdomain' => true, 'range' => array('lucabert.de', 'lucabert.com')),
        'date' => array('type' => myMVC_BaseAction::$VALIDATE_DATETIME, 'empty' => false, 'format' => _('%m/%d/%Y')),
        'time' => array('type' => myMVC_BaseAction::$VALIDATE_DATETIME, 'empty' => false, 'format' => _('%H:%M:%S')),
        'datetime' => array('type' => myMVC_BaseAction::$VALIDATE_DATETIME, 'empty' => true, 'format' => _('%m/%d/%Y %H:%M:%S')),
        'testExt' => array('type' => myMVC_BaseAction::$VALIDATE_EXTERN, 'checkFunction' => 'MainAction::check'),
        'testDB' => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'rangesqli' => 'SELECT firstname FROM test WHERE last_use IS NULL'),
        'testLogin' => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'maxlen' => 20, 'minlen' => 5, 'empty' => false,
                                                   'comparedb' => array('query' => "SELECT COUNT(*) FROM test WHERE firstname = {checkValue}",
                                                                        'compareType' => '==', 'compareValue' => 0)),
        'testPass1' => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'maxlen' => 20, 'minlen' => 5, 'empty' => true),
        'testPass2' => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'maxlen' => 20, 'minlen' => 5, 'empty' => true,
                                                   'comparefield' => array('field' => 'testPass1', 'compareType' => '==')),
    );
// Names of the fields (for the error messages)
    $names = array(
        'text' => _('Text field'),
        'bool' => _('Bool field'),
        'number' => _('Number field'),
        'email' => _('E-Mail address'),
        'web' => _('URL'),
        'domain' => _('Domain'),
        'date' => _('Date'),
        'time' => _('Time'),
        'datetime' => _('Date+Time'),
        'testExt' => _('Test'),
        'testDB' => _('Test DB'),
        'testLogin' => _('Test Login'),
        'testPass1' => _('Test Password'),
        'testPass2' => _('Repeat Password'),
    );
// Custom error messages for every field.
// You can define an error message for every not passed check.
// In myMVC_BaseAction you have many $ERROR_ string to define
// these errors.
// In the message (please note own you can use GetText in this context!)
// the TAGs {field} and {checkValue} will be substituted with the field
// name (see $names) and his value.
// For the $VALIDATE_EXTERN-check you have {externError}, too, containing
// an error given from the external check function.
    $errorMsgs = array(
        'text'      => array(myMVC_BaseAction::$ERROR_MINLEN  => sprintf(_("%s must be at least %s char long"),
                                               '{field}', '{checkValue}'),
                             myMVC_BaseAction::$ERROR_EMPTY   => sprintf(_("%s cannot be empty"),
                                               '{field}')),
        'bool'      => array(myMVC_BaseAction::$ERROR_NOTBOOL => sprintf(_("%s must be a boolean value"),
                                               '{field}'),
                             myMVC_BaseAction::$ERROR_EMPTY   => sprintf(_("%s cannot be empty"),
                                               '{field}')),
        'number'    => array(myMVC_BaseAction::$ERROR_NOTNUMBER => sprintf(_("%s must be a number"),
                                               '{field}'),
                             myMVC_BaseAction::$ERROR_EMPTY   => sprintf(_("%s cannot be empty"),
                                               '{field}'),
                             myMVC_BaseAction::$ERROR_MIN   => sprintf(_("%s must be greater than %s"),
                                               '{field}', '{checkValue}'),
                             myMVC_BaseAction::$ERROR_MAX   => sprintf(_("%s must be lesser than %s"),
                                               '{field}', '{checkValue}'),
                             myMVC_BaseAction::$ERROR_RANGE => sprintf(_("%s is not in the allowed range"),
                                               '{field}')),
        'email'     => array(myMVC_BaseAction::$ERROR_NOTEMAIL => sprintf(_("%s is not a valid E-Mail address"),
                                               '{field}'),
                             myMVC_BaseAction::$ERROR_EMPTY   => sprintf(_("%s cannot be empty"),
                                               '{field}'),
                             myMVC_BaseAction::$ERROR_RANGE => sprintf(_("%s is not in the allowed range"),
                                               '{field}')),
        'web'       => array(myMVC_BaseAction::$ERROR_NOTWEB => sprintf(_("%s is not a valid URL"),
                                               '{field}'),
                             myMVC_BaseAction::$ERROR_EMPTY   => sprintf(_("%s cannot be empty"),
                                               '{field}')),
        'domain'    => array(myMVC_BaseAction::$ERROR_NOTWEB => sprintf(_("%s is not a valid internet domain"),
                                               '{field}'),
                             myMVC_BaseAction::$ERROR_EMPTY   => sprintf(_("%s cannot be empty"),
                                               '{field}'),
                             myMVC_BaseAction::$ERROR_RANGE => sprintf(_("%s is not in the allowed range"),
                                               '{field}')),
        'date'      => array(myMVC_BaseAction::$ERROR_NOTDATETIME => sprintf(_("%s is not a valid date"),
                                               '{field}'),
                             myMVC_BaseAction::$ERROR_EMPTY   => sprintf(_("%s cannot be empty"),
                                               '{field}')),
        'time'      => array(myMVC_BaseAction::$ERROR_NOTDATETIME => sprintf(_("%s is not a valid time"),
                                               '{field}'),
                             myMVC_BaseAction::$ERROR_EMPTY   => sprintf(_("%s cannot be empty"),
                                               '{field}')),
        'datetime'  => array(myMVC_BaseAction::$ERROR_NOTDATETIME => sprintf(_("%s is not a valid timestamp"),
                                               '{field}'),
                             myMVC_BaseAction::$ERROR_EMPTY   => sprintf(_("%s cannot be empty"),
                                               '{field}')),
        'testExt'   => array(myMVC_BaseAction::$ERROR_INVALID => sprintf(_("%s is invalid (%s)"),
                                               '{field}', '{externError}')),
        'testDB'    => array(myMVC_BaseAction::$ERROR_NOTNUMBER => sprintf(_("%s must be a number"),
                                               '{field}'),
                             myMVC_BaseAction::$ERROR_EMPTY   => sprintf(_("%s cannot be empty"),
                                               '{field}'),
                             myMVC_BaseAction::$ERROR_RANGE => sprintf(_("%s is not in the allowed range"),
                                               '{field}')),
        'testLogin' => array(myMVC_BaseAction::$ERROR_MINLEN  => sprintf(_("%s must be at least %s char long"),
                                               '{field}', '{checkValue}'),
                             myMVC_BaseAction::$ERROR_MAXLEN  => sprintf(_("%s cannot be longer the %s char"),
                                               '{field}', '{checkValue}'),
                             myMVC_BaseAction::$ERROR_COMPAREDB  => sprintf(_("%s is already used"),
                                               '{field}'),
                             myMVC_BaseAction::$ERROR_EMPTY   => sprintf(_("%s cannot be empty"),
                                               '{field}')),
        'testPass1' => array(myMVC_BaseAction::$ERROR_MINLEN  => sprintf(_("%s must be at least %s char long"),
                                               '{field}', '{checkValue}'),
                             myMVC_BaseAction::$ERROR_MAXLEN  => sprintf(_("%s cannot be longer the %s char"),
                                               '{field}', '{checkValue}')),
        'testPass2' => array(myMVC_BaseAction::$ERROR_MINLEN  => sprintf(_("%s must be at least %s char long"),
                                               '{field}', '{checkValue}'),
                             myMVC_BaseAction::$ERROR_MAXLEN  => sprintf(_("%s cannot be longer the %s char"),
                                               '{field}', '{checkValue}'),
                             myMVC_BaseAction::$ERROR_COMPAREFIELD => sprintf(_("%s is not equal to test password"),
                                               '{field}'),
                                               ),
    );
    return parent::validateInput($definitions, $names, $errorMsgs);
  }

// External check function.
// The field must be '123', otherwise the check fails
// and 'Ghhh' will be as error returned (see TAG {externError})
  public static function check($request, $value, &$errMsg)
  {
    if($value != '123')
    {
      $errMsg = 'Ghhh';
      return false;
    }
    return true;
  }

// This action always call MainView
  /**
   * Perform the Action
   * 
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function perform()
  {
    $response = new myMVC_HttpResponse();
    $response->setNext('MainView');
    return $response;
  }

  /**
   * Called if user data are not valid (check with validate)
   * 
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function manageInvalidData()
  {
    $response = new myMVC_HttpResponse();
    $response->setNext('MainView');
    return $response;
  }
}
?>
