<?php
/**
 * Example to use myMVC.
 * This program just demonstrate how to create
 * a login form, authenticate user, check data from
 * a form and read data from database.
 * 
 * Author: Luca Bertoncello <lucabert@lucabert.de>
 */

/**
 * Using database.
 * This Action check if a given string is valid (at least 3 chars)
 * and display some data from database.
 */
class TestDBAction extends myMVC_BaseAction
{
  public function __construct($request, $session)
  {
    parent::__construct($request, $session);
// Define the validatate-function
    $this->setCheckFunction('POST', 'validatePost');
  }

  /**
   * Returns the token used to identify the current Action
   * 
   * @return string                 The token
   */
  public static function getActionToken()
  {
    return 'testDB';
  }

  /**
   * Returns whether this action should appear in the menu (created by controller)
   * 
   * @return boolean                true if the action should appear in menu
   */
  public static function isMenuAction()
  {
// Yes, this Action will be called from menu!
    return true;
  }

  /**
   * Returns the string to be displayed in the menu to identify the current action.
   * Only necessary if isMenuAction() returns true
   * 
   * @return string                 The menu item
   */
  public static function getActionMenuName()
  {
    return _('Test database');
  }

  /**
   * Returns whether the current user is allowed to call the current Action
   * 
   * @return boolean                true if the user may call this Action, false otherwise
   */
  public static function hasUserRightForAction()
  {
    return true;
  }

  /**
   * Validate the data from user, coming as POST (form submit).
   * 
   * @return boolean                true if data are valid, false otherwise
   */
  public function validatePost()
  {
// User's string has to be at least 3 chars long
    $definitions = array(
        'text' => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'minlen' => 3, 'empty' => false),
    );
    $names = array(
        'text' => _('Text to search'),
    );
    $errorMsgs = array(
        'text'      => array(myMVC_BaseAction::$ERROR_MINLEN  => sprintf(_("%s must be at least %s char long"),
                                               '{field}', '{checkValue}'),
                             myMVC_BaseAction::$ERROR_EMPTY   => sprintf(_("%s cannot be empty"),
                                               '{field}')),
    );
    return parent::validateInput($definitions, $names, $errorMsgs);
  }

  /**
   * Perform the Action
   * 
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function perform()
  {
    $response = new myMVC_HttpResponse();
// TestDBView will always be called...
    $response->setNext('TestDBView');

    if($this->request->getRequestMethod() == 'POST')
    {
// If the user used the form (Submit-Type: POST),
// get the records with lastname like the given string
      $this->saveParameterForNextPage('countSearched',
                  testDB::callCounter('getCountByLastname', array('lastname' => $this->request->getParameter('text'))));
      $this->saveParameterForNextPage('searched',
                  testDB::callFinder('findByLastname', array('lastname' => $this->request->getParameter('text'))));
    }

// The some other data from the database
    $this->getData();

    return $response;
  }

  /**
   * Called if user data are not valid (check with validate)
   * 
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function manageInvalidData()
  {
// This method will be called if "text" is invalid.
// It just display the normal page and some data from database,
// but does NOT search in the database (since given data are NOT valid)
    $response = new myMVC_HttpResponse();
    $response->setNext('TestDBView');

    $this->getData();

    return $response;
  }

  protected function getData()
  {
// Change the last use timestamp for the Record with ID 1
    $t = testDB::findByPK(array('id' => 1));
    if($t !== NULL)
    {
      $t->setLastUse($t->now());
      $t->save();
    }
    $this->saveParameterForNextPage('id1', $t);

// Get all records
    $this->saveParameterForNextPage('countAll', testDB::getCountAll());
    $this->saveParameterForNextPage('all', testDB::findAll());

// Test insert, with unique constraints
    $x = new testDB();
    $x->setFirstname('sdfg');
    $x->setLastname('dfgggg');
    $x->setTest(5);
    if($x->isRecordUnique())
    {
      $this->saveParameterForNextPage('inserted', true);
      $x->save();
    }
    else
      $this->saveParameterForNextPage('inserted', false);
  }
}
?>
